package com.cultural.heritage.controller.address;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.address.AddressAddRequest;
import com.cultural.heritage.model.dto.address.AddressUpdateRequest;
import com.cultural.heritage.model.entity.Address;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.service.address.AddressService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping("/address")
@Slf4j
@Tag(name = "地址信息模块")
public class AddressController {


    @Resource
    private AddressService addressService;



    @Resource
    private UserService userService;


    @Resource
    private CommonService commonService;


    /**
     * 添加地址信息
     * @param addressAddRequest  地址添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户添加地址信息", description = "参数：地址添加请求体，权限：所有人，方法名：addAddress")
    public BaseResponse<Long> addAddress(@RequestBody AddressAddRequest addressAddRequest, HttpServletRequest request) {
        if (addressAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Address address = commonService.copyProperties(addressAddRequest, Address.class);
        address.setUserId(userId);
        // 处理之前的默认地址
        if (address.getIsDefault() == 1) {
            LambdaUpdateWrapper<Address> lambdaUpdateWrapper  = new LambdaUpdateWrapper<>();
            lambdaUpdateWrapper.eq(Address::getUserId, userId).eq(Address::getIsDefault, 1).set(Address::getIsDefault, 0);
            addressService.update(lambdaUpdateWrapper);
        }
        addressService.validAddress(address, false);
        addressService.save(address);
        return ResultUtils.success(address.getId());
    }



    /**
     * 删除地址信息
     * @param deleteRequest   地址删除请求体
     * @return  是否成功删除
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户删除地址信息", description = "参数：地址删除请求体，权限：所有人，方法名：delAddress")
    public BaseResponse<Boolean> delAddress(@RequestBody CommonRequest deleteRequest, HttpServletRequest request) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = deleteRequest.getId();
        addressService.removeById(id);
        return ResultUtils.success(true);
    }




    /**
     * 更新地址信息
     * @param addressUpdateRequest  地址更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "小程序端用户更新地址信息", description = "参数：地址更新请求体，权限：所有人，方法名：updateAddress")
    public BaseResponse<Boolean> updateAddress(@RequestBody AddressUpdateRequest addressUpdateRequest, HttpServletRequest request) {
        if (addressUpdateRequest == null || addressUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Address address = commonService.copyProperties(addressUpdateRequest, Address.class);
        address.setUserId(userId);
        // 处理之前的默认地址
        if (address.getIsDefault() == 1) {
            LambdaUpdateWrapper<Address> lambdaUpdateWrapper  = new LambdaUpdateWrapper<>();
            lambdaUpdateWrapper.eq(Address::getUserId, userId).eq(Address::getIsDefault, 1).set(Address::getIsDefault, 0);
            addressService.update(lambdaUpdateWrapper);
        }
        addressService.validAddress(address, true);
        addressService.updateById(address);
        return ResultUtils.success(true);
    }




    /**
     * 查询用户地址信息
     * @return 用户地址列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端查询用户地址信息", description = "参数：无，权限：所有人，方法名:listAddress")
    public BaseResponse<List<Address>> listAddress(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        LambdaQueryWrapper<Address> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Address::getUserId, userId);
        List<Address> addressList = addressService.list(queryWrapper);
        Collections.reverse(addressList);
        return ResultUtils.success(addressList);
    }



}
