package com.cultural.heritage.controller.clothes;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clothes.ClothesAddRequest;
import com.cultural.heritage.model.dto.clothes.ClothesQueryRequest;
import com.cultural.heritage.model.dto.clothes.ClothesUpdateRequest;
import com.cultural.heritage.model.dto.facelift.ClothesUrlObj;
import com.cultural.heritage.model.dto.facelift.GarmentObj;
import com.cultural.heritage.model.dto.facelift.LogoUrlObj;
import com.cultural.heritage.model.dto.facelift.ModelObj;
import com.cultural.heritage.model.entity.Clothes;
import com.cultural.heritage.model.vo.clothes.ClothesLabelVO;
import com.cultural.heritage.model.vo.clothes.ClothesVO;
import com.cultural.heritage.service.clothes.ClothesService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.utils.DecoderUtils;
import com.cultural.heritage.utils.Sign;
import com.google.gson.Gson;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/clothes")
@Slf4j
@Tag(name = "服装租赁产品管理模块")
public class ClothesController {

    @Resource
    private ClothesService clothesService;


    @Resource
    private CommonService commonService;


    @Resource
    private RedisTemplate<String, Clothes> clothesRedisTemplate;


    private static final String CLOTHES_KEY = "clothes:";



    /**
     * Web端管理员添加服装租赁产品
     * @param clothesAddRequest 服装租赁产品添加请求体
     * @return 服装租赁产品id
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加服装租赁产品", description = "参数：服装租赁产品添加请求体，权限：管理员(admin, boss)，方法名：addClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> addClothes(@RequestBody ClothesAddRequest clothesAddRequest) {
        if (clothesAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Clothes clothes = commonService.copyProperties(clothesAddRequest, Clothes.class);
        // 校验
        clothesService.validClothes(clothes, false);
        clothesService.save(clothes);

        // 缓存商品详情到 Redis
        String clothesCacheKey = CLOTHES_KEY + clothes.getId();
        clothesRedisTemplate.opsForValue().set(clothesCacheKey, clothes, 1, TimeUnit.HOURS); // 缓存商品详情，1小时

        return ResultUtils.success(clothes.getId());
    }


    /**
     * Web端管理员更新服装租赁产品
     * @param clothesUpdateRequest 服装租赁产品更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新服装租赁产品", description = "参数：服装租赁产品更新请求体，权限：管理员(admin, boss)，方法名：updateClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateClothes(@RequestBody ClothesUpdateRequest clothesUpdateRequest) {
        if (clothesUpdateRequest == null || clothesUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新服装租赁产品
        Clothes clothes = commonService.copyProperties(clothesUpdateRequest, Clothes.class);
        clothesService.validClothes(clothes, true);
        clothesService.updateById(clothes);

        // 清除 Redis 缓存
        String cacheKey = CLOTHES_KEY + clothes.getId();
        clothesRedisTemplate.delete(cacheKey);

        return ResultUtils.success(true);
    }


    /**
     * Web端管理员根据id删除服装租赁产品
     * @param commonRequest 服装租赁产品id
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员根据id删除服装租赁产品", description = "参数：服装租赁产品id，权限：管理员(admin, boss)，方法名：deleteClothes")
    public BaseResponse<Boolean> deleteClothes(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        clothesService.removeById(commonRequest.getId());

        // 删除缓存中的相应数据
        String cacheKey = CLOTHES_KEY + commonRequest.getId();
        clothesRedisTemplate.delete(cacheKey);

        return ResultUtils.success(true);
    }



    /**
     * web端用户根据id查询服装主要信息
     * @param commonRequest 服装id
     * @return  服装信息
     */
    @PostMapping("/query/id")
    @Operation(summary = "web端用户根据id查询服装主要信息", description = "参数：服装id，权限：所有人，方法名：webGetClothesById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<ClothesVO> webGetClothesById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        return ResultUtils.success(clothesService.getClothesById(commonRequest.getId()));
    }




    /**
     * 小程序端用户根据id查询服装
     */
    @PostMapping("/get/id")
    @Operation(summary = "小程序端用户根据id查询服装", description = "参数：服装id，权限：所有人，方法名：getClothesById")
    public BaseResponse<ClothesVO> getClothesById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        Long id = commonRequest.getId();
        // Redis缓存的key
        String cacheKey = CLOTHES_KEY + id;
        // 先从Redis中获取商品信息
        ClothesVO clothesVO;
        Clothes clothes = clothesRedisTemplate.opsForValue().get(cacheKey);
        // 如果缓存命中，直接返回
        if (clothes != null) {
            clothesVO = commonService.copyProperties(clothes, ClothesVO.class);
            return ResultUtils.success(clothesVO);
        }
        Clothes clothesInfo = clothesService.getClothesInfoById(id);
        clothesRedisTemplate.opsForValue().set(cacheKey, clothesInfo, 1, TimeUnit.HOURS); // 缓存1小时
        clothesVO = commonService.copyProperties(clothesInfo, ClothesVO.class);
        return ResultUtils.success(clothesVO);
    }



    /**
     * 小程序端用户根据id查询服装（不包含富文本）
     */
    @PostMapping("/get/label/id")
    @Operation(summary = "小程序端用户根据id查询服装（不包含富文本）", description = "参数：服装id，权限：所有人，方法名：getClothesLabelById")
    public BaseResponse<ClothesLabelVO> getClothesLabelById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        Long id = commonRequest.getId();
        // Redis缓存的key
        String cacheKey = CLOTHES_KEY + id;
        // 先从Redis中获取商品信息
        ClothesLabelVO clothesLabelVO;
        Clothes clothes = clothesRedisTemplate.opsForValue().get(cacheKey);
        // 如果缓存命中，直接返回
        if (clothes != null) {
            clothesLabelVO = commonService.copyProperties(clothes, ClothesLabelVO.class);
            return ResultUtils.success(clothesLabelVO);
        }
        Clothes clothesInfo = clothesService.getClothesInfoById(id);
        clothesRedisTemplate.opsForValue().set(cacheKey, clothesInfo, 1, TimeUnit.HOURS); // 缓存1小时
        clothesLabelVO = commonService.copyProperties(clothesInfo, ClothesLabelVO.class);
        return ResultUtils.success(clothesLabelVO);
    }




    /**
     * Web端管理员分页查询服装
     * @param clothesQueryRequest 查询服装请求体
     * @return 服装信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询服装", description = "参数：服装查询请求体，排序字段 sortField: price inventory id，权限：管理员(admin, boss)，方法名：listClothesByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ClothesVO>> listClothesByPage(@RequestBody ClothesQueryRequest clothesQueryRequest) {
        long current = clothesQueryRequest.getCurrent();
        long pageSize = clothesQueryRequest.getPageSize();
        QueryWrapper<Clothes> clothesQueryWrapper = clothesService.getQueryWrapper(clothesQueryRequest);
        Page<Clothes> page = clothesService.page(new Page<>(current, pageSize), clothesQueryWrapper);
        List<Clothes> clothesList = page.getRecords();
        // 处理富文本字段
        for (Clothes clothes : clothesList) {
            if (StringUtils.isNotBlank(clothes.getRichText())) clothes.setRichText(DecoderUtils.decodeText(clothes.getRichText()));
        }
        List<ClothesVO> clothesVOS = commonService.convertList(clothesList, ClothesVO.class);
        Page<ClothesVO> clothesPageVOPage = new Page<>();
        clothesPageVOPage.setRecords(clothesVOS);
        clothesPageVOPage.setTotal(page.getTotal());
        clothesPageVOPage.setSize(page.getSize());
        clothesPageVOPage.setCurrent(page.getCurrent());
        clothesPageVOPage.setPages(page.getPages());

        return ResultUtils.success(clothesPageVOPage);
    }






    /**
     * Web端管理员批量删除服装
     * @param commonDelBatchRequest 批量删除请求体
     * @return 是否批量删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除服装", description = "参数：服装批量删除请求体，权限：管理员(admin, boss)，方法名：delBatchClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchClothes(@RequestBody CommonBatchRequest commonDelBatchRequest) {
        if (commonDelBatchRequest == null || CollectionUtils.isEmpty(commonDelBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonDelBatchRequest.getIdList();
        // 批量删除服装
        boolean result = clothesService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装批量删除失败");

        // 批量删除缓存中的数据
        for (Long id : idList) {
            String cacheKey = CLOTHES_KEY + id;
            clothesRedisTemplate.delete(cacheKey);  // 删除每个服装的缓存
        }

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员上(下)架服装
     * @param commonRequest 服装id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架服装", description = "参数：服装id，权限：管理员(admin, boss)，方法名：updateClothesShelvesStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateClothesShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取当前服装的上(下)架状态
        Long id = commonRequest.getId();
        Clothes clothes = clothesService.getById(id);
        ThrowUtils.throwIf(clothes == null, ErrorCode.OPERATION_ERROR, "服装不存在");
        Integer status = clothes.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<Clothes> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        clothesService.update(updateWrapper);
        return ResultUtils.success(true);
    }





    /**
     * 小程序端用户调用换装api
     * @param modelURL  模特Url
     * @param clothesURL  服装URL
     * @return
     */
    @GetMapping("/facelift")
    @Operation(summary = "小程序端用户调用换装api", description = "参数：服装id，权限：管理员(admin, boss)，方法名：updateClothesShelvesStatus")
    @AuthCheck(mustRole = UserConstant.DEFAULT_ROLE)
    public BaseResponse<String> invokeChangeClothesApi(@RequestParam String modelURL, @RequestParam String clothesURL) {
        if (StringUtils.isBlank(modelURL) || StringUtils.isBlank(clothesURL)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不能为空");
        }
        Map<String, Object> param = new HashMap<>();
        param.put("req_key", "dressing_diffusion");
        ModelObj modelObj = ModelObj.builder().id("1").url(modelURL).build();
        param.put("model", modelObj);
        ClothesUrlObj clothesUrlObj = ClothesUrlObj.builder().url(clothesURL).build();
        List<ClothesUrlObj> clothesUrlObjList = new ArrayList<>();
        clothesUrlObjList.add(clothesUrlObj);
        GarmentObj garmentObj = GarmentObj.builder().id("1").data(clothesUrlObjList).build();
        param.put("garment", garmentObj);
        param.put("return_url", true);
        LogoUrlObj logoUrlObj = LogoUrlObj.builder().add_logo(false).position(0).language(0).logo_text_content("这里是明水印内容").build();
        param.put("logo_url", logoUrlObj);
        // 使用 Gson 将 Map 转换为 byte[]
        Gson gson = new Gson();
        byte[] body = gson.toJson(param).getBytes(StandardCharsets.UTF_8);
        String responseBody = null;
        try {
            responseBody = Sign.invokeChangeClothesApi(body);
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject jsonObject = JSONObject.parseObject(responseBody);
        JSONArray imageUrls = jsonObject.getJSONObject("data").getJSONArray("image_urls");
        return ResultUtils.success(imageUrls.getString(0));
    }









//
//    // Redis 查询模拟接口
//    @PostMapping("/query/redis")
//    public String simulateRedisQuery(@RequestParam int count) throws InterruptedException {
//        long startTime = System.currentTimeMillis();
//
//        CountDownLatch latch = new CountDownLatch(count);
//        for (int i = 1; i <= count; i++) {
//            queryFromRedis(2L, latch);
//        }
//        latch.await(); // 等待所有线程完成
//
//        long endTime = System.currentTimeMillis();
//        return "Redis Query completed in: " + (endTime - startTime) + " ms";
//    }
//
//    // MySQL 查询模拟接口
//    @PostMapping("/query/mysql")
//    public String simulateMySQLQuery(@RequestParam int count) throws InterruptedException {
//        long startTime = System.currentTimeMillis();
//
//        CountDownLatch latch = new CountDownLatch(count);
//        for (int i = 1; i <= count; i++) {
//            queryFromMySQL(2L, latch);
//        }
//        latch.await(); // 等待所有线程完成
//
//        long endTime = System.currentTimeMillis();
//        return "MySQL Query completed in: " + (endTime - startTime) + " ms";
//    }
//
//    private static int redisCnt = 0;
//    private static int mysqlCnt = 0;
//
//    @Async
//    public void queryFromRedis(Long id, CountDownLatch latch) {
//        String cacheKey = "clothes:" + id;
//        Clothes clothes = clothesRedisTemplate.opsForValue().get(cacheKey);
//        System.out.println("缓存 ==============》 " + redisCnt ++ );
//        if (clothes == null) {
//            clothes = clothesService.getById(id);
//        }
//        latch.countDown(); // 查询结束，释放计数器
//    }
//
//    // 通过 MySQL 获取商品
//    @Async
//    public void queryFromMySQL(Long id, CountDownLatch latch) {
//        Clothes clothes = clothesService.getById(id);
//        System.out.println("缓存 ==============》 " + mysqlCnt ++ );
//        latch.countDown(); // 查询结束，释放计数器
//    }
//



}
