package com.cultural.heritage.controller.order;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clothesRentOrder.ClothesRentOrderAddRequest;
import com.cultural.heritage.model.dto.clothesRentOrder.ClothesRentOrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderStatusUpdateRequest;
import com.cultural.heritage.model.dto.snapshot.ClothesSnapshot;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.vo.clothesRentOrder.ClothesRentOrderVO;
import com.cultural.heritage.service.address.ContactsService;
import com.cultural.heritage.service.clothes.ClothesService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.order.ClothesRentOrderService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.OrderNumberUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/clothesRent")
@Slf4j
@Tag(name = "服装租赁订单管理模块")
public class ClothesRentOrderController {



    @Resource
    private ClothesRentOrderService clothesRentOrderService;


    @Resource
    private ClothesService clothesService;


    @Resource
    private UserService userService;


    @Resource
    private ContactsService contactsService;


    @Resource
    private CommonService commonService;



    /**
     * 小程序端用户创建服装租赁订单
     * @param clothesRentOrderAddRequest 服装租赁订单添加请求体
     * @return  订单id
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户创建服装租赁订单", description = "参数：服装租赁订单添加请求体, 权限：所有人，方法名：addClothesRentOrder")
    public BaseResponse<Long> addClothesRentOrder(@RequestBody ClothesRentOrderAddRequest clothesRentOrderAddRequest, HttpServletRequest request) {
        if (clothesRentOrderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取用户id
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        clothesRentOrderService.validClothesRentOrder(clothesRentOrderAddRequest);
        // 获取服装信息
        Long clothesId = clothesRentOrderAddRequest.getClothesId();
        // 校验当前服装是否已被租赁
        clothesRentOrderService.validIsBeenRented(clothesId);
        Clothes clothes = clothesService.getById(clothesId);
        ThrowUtils.throwIf(clothes == null || clothes.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "服装已被下架或者不存在");
        ClothesSnapshot clothesSnapshot = commonService.copyProperties(clothes, ClothesSnapshot.class);
        // 获取联系人信息
        Long contactsId = clothesRentOrderAddRequest.getContactsId();
        Contacts contacts = contactsService.getById(contactsId);
        ThrowUtils.throwIf( contacts == null, ErrorCode.OPERATION_ERROR, "联系人信息不存在");
        ContactsSnapshot contactsSnapshot = commonService.copyProperties(contacts, ContactsSnapshot.class);
        // 校验租赁天数
        Integer rentDays = clothesRentOrderAddRequest.getRentDays();
        ThrowUtils.throwIf(rentDays > clothes.getPeriod(), ErrorCode.OPERATION_ERROR, "超出当前服装的最大租赁天数");
        // 计算订单总金额
        BigDecimal price = clothes.getPrice();
        BigDecimal totalAmount = price.multiply(BigDecimal.valueOf(rentDays));
        // 生成订单号
        String orderNumber = OrderNumberUtils.generateOrderId();
        // 封装成服装租赁订单
        ClothesRentOrder clothesRentOrder = new ClothesRentOrder();
        clothesRentOrder.setUserId(userId);
        clothesRentOrder.setOrderNumber(orderNumber);
        clothesRentOrder.setOrderStatus(OrderStatusConstant.PENDING_PAYMENT);
        clothesRentOrder.setClothesSnapshot(clothesSnapshot);
        clothesRentOrder.setContactsSnapshot(contactsSnapshot);
        clothesRentOrder.setRentDays(rentDays);
        clothesRentOrder.setTotalAmount(totalAmount);

        clothesRentOrderService.save(clothesRentOrder);
        clothesRentOrderService.sendClothesRentOrderMessage(clothesRentOrder.getId());

        return ResultUtils.success(clothesRentOrder.getId());

    }





    /**
     * 小程序端用户取消服装租赁订单
     * @param commonRequest  订单id
     * @return   是否取消成功
     */
    @PostMapping ("/cancel/id")
    @Operation(summary = "小程序端用户取消服装租赁订单", description = "参数：订单id，权限：所有人，方法名：cancelClothesRentOrderById")
    public BaseResponse<Boolean> cancelClothesRentOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        ClothesRentOrder clothesRentOrder = clothesRentOrderService.getById(id);
        ThrowUtils.throwIf(clothesRentOrder == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ThrowUtils.throwIf(!clothesRentOrder.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.SYSTEM_ERROR, "订单状态错误");
        UpdateWrapper<ClothesRentOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id).set("orderStatus", OrderStatusConstant.TRANSACTION_CLOSED);
        clothesRentOrderService.update(updateWrapper);
        return ResultUtils.success(true);
    }







    /**
     * Web端管理员分页查询服装租赁订单
     * @param clothesRentOrderQueryRequest  服装租赁订单查询请求体
     * @return  服装租赁订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "Web端管理员分页查询服装租赁订单", description = "参数：服装租赁订单查询请求体, 权限：管理员(admin, boss)，方法名：listClothesRentOrderByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ClothesRentOrderVO>> listClothesRentOrderByPage(@RequestBody ClothesRentOrderQueryRequest clothesRentOrderQueryRequest) {
        if (clothesRentOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = clothesRentOrderQueryRequest.getCurrent();
        long pageSize = clothesRentOrderQueryRequest.getPageSize();
        QueryWrapper<ClothesRentOrder> queryWrapper = clothesRentOrderService.getQueryWrapper(clothesRentOrderQueryRequest);
        Page<ClothesRentOrder> page = clothesRentOrderService.page(new Page<>(current, pageSize), queryWrapper);
        // 封装成服装租赁订单VO
        List<ClothesRentOrder> clothesRentOrderList = page.getRecords();
        List<ClothesRentOrderVO> clothesRentOrderVOS = commonService.convertList(clothesRentOrderList, ClothesRentOrderVO.class);
        // 填充userName
        List<User> userList = commonService.findByFieldInTargetField(clothesRentOrderVOS, userService, ClothesRentOrderVO::getUserId, "id");
        Map<Long, String> map = new HashMap<>();
        for (User user : userList) {
            map.put(user.getId(), user.getUserName());
        }
        for (ClothesRentOrderVO clothesRentOrderVO : clothesRentOrderVOS) {
            Long userId = clothesRentOrderVO.getUserId();
            String userName = map.get(userId);
            clothesRentOrderVO.setUserName(userName);
        }
        Page<ClothesRentOrderVO> voPage = new Page<>();
        voPage.setRecords(clothesRentOrderVOS);
        voPage.setTotal(page.getTotal());
        voPage.setPages(page.getSize());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        return ResultUtils.success(voPage);
    }




    /**
     * 小程序端用户查询服装租赁订单
     * @return  当前用户的订单列表
     */
    @PostMapping("/list/my")
    @Operation(summary = "小程序端用户查询服装租赁订单", description = "参数：无, 权限：所有人，方法名：listMyClothesRentOrder")
    public BaseResponse<List<ClothesRentOrderVO>> listMyClothesRentOrder(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<ClothesRentOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<ClothesRentOrder> clothesRentOrderList = clothesRentOrderService.list(queryWrapper);
        // 封装成写真预约订单VO
        List<ClothesRentOrderVO> clothesRentOrderVOS = commonService.convertList(clothesRentOrderList, ClothesRentOrderVO.class);
        for (ClothesRentOrderVO clothesRentOrderVO : clothesRentOrderVOS) {
            clothesRentOrderVO.setUserId(userId);
        }
        Collections.reverse(clothesRentOrderVOS);
        return ResultUtils.success(clothesRentOrderVOS);
    }





    /**
     * 小程序端用户根据id删除服装租赁订单
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户根据id删除服装租赁订单", description = "参数：无, 权限：所有人，方法名：delClothesRentOrderById")
    public BaseResponse<Boolean> delClothesRentOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        ClothesRentOrder clothesRentOrder = clothesRentOrderService.getById(id);
        ThrowUtils.throwIf(clothesRentOrder == null || !clothesRentOrder.getOrderStatus().equals(OrderStatusConstant.TRANSACTION_CLOSED), ErrorCode.OPERATION_ERROR, "订单不存在或状态错误");
        clothesRentOrderService.removeById(id);
        return ResultUtils.success(true);
    }




    /**
     * 小程序端用户根据id查询服装租赁订单
     * @param commonRequest 订单id
     * @return  是否删除成功
     */
    @PostMapping("/get/id")
    @Operation(summary = "小程序端用户根据id查询服装租赁订单", description = "参数：订单id, 权限：所有人，方法名：queryClothesRentOrderById")
    public BaseResponse<ClothesRentOrderVO> queryClothesRentOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long id = commonRequest.getId();
        ClothesRentOrder clothesRentOrder = clothesRentOrderService.getById(id);
        ThrowUtils.throwIf(clothesRentOrder == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ClothesRentOrderVO clothesRentOrderVO = commonService.copyProperties(clothesRentOrder, ClothesRentOrderVO.class);
        clothesRentOrderVO.setUserName(loginUser.getUserName());
        return ResultUtils.success(clothesRentOrderVO);
    }





    /**
     * Web端管理员更新服装租赁订单状态
     * @param orderStatusUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update/orderStatus")
    @Operation(summary = "Web端管理员更新服装租赁订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateClothesRentOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateClothesRentOrderStatus(@RequestBody OrderStatusUpdateRequest orderStatusUpdateRequest) {
        if (orderStatusUpdateRequest == null || orderStatusUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderStatusUpdateRequest.getId();
        String orderStatus = orderStatusUpdateRequest.getOrderStatus();
        ThrowUtils.throwIf(StringUtils.isBlank(orderStatus), ErrorCode.OPERATION_ERROR, "订单状态参数为空");
        UpdateWrapper<ClothesRentOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        if (orderStatus.equals(OrderStatusConstant.PENDING_DELIVERY)) updateWrapper.set("pickupTime", DateUtil.now());
        boolean result = clothesRentOrderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单状态不存在或订单状态更新失败");

        // 如果订单状态为待收货，就发送消息队列，开启服装租赁倒计时
        ClothesRentOrder clothesRentOrder = clothesRentOrderService.getById(id);
        if (orderStatus.equals(OrderStatusConstant.PENDING_DELIVERY)) {
            clothesRentOrderService.sendClothesRentPeriodMessage(id, clothesRentOrder.getRentDays());
        }
        return ResultUtils.success(true);
    }

}
