package com.cultural.heritage.service.order.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.ClothesRentOrderMapper;
import com.cultural.heritage.model.dto.clothesRentOrder.ClothesRentOrderAddRequest;
import com.cultural.heritage.model.dto.clothesRentOrder.ClothesRentOrderQueryRequest;
import com.cultural.heritage.model.entity.ClothesRentOrder;
import com.cultural.heritage.service.order.ClothesRentOrderService;
import com.cultural.heritage.utils.MultiDelayMessage;
import com.cultural.heritage.utils.SqlUtils;
import com.cultural.heritage.utils.TimeSplitterUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ClothesRentOrderServiceImpl extends ServiceImpl<ClothesRentOrderMapper, ClothesRentOrder> implements ClothesRentOrderService {


    @Resource
    private RabbitTemplate rabbitTemplate;


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<ClothesRentOrder> getQueryWrapper(ClothesRentOrderQueryRequest clothesRentOrderQueryRequest) {
        String id = clothesRentOrderQueryRequest.getId();
        String orderNumber = clothesRentOrderQueryRequest.getOrderNumber();
        String orderStatus = clothesRentOrderQueryRequest.getOrderStatus();
        String name = clothesRentOrderQueryRequest.getName();
        String phone = clothesRentOrderQueryRequest.getPhone();
        String sortField = clothesRentOrderQueryRequest.getSortField();
        String sortOrder = clothesRentOrderQueryRequest.getSortOrder();
        QueryWrapper<ClothesRentOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.eq(StringUtils.isNotBlank(orderNumber), "orderNumber", orderNumber);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.like(StringUtils.isNotBlank(name), "JSON_UNQUOTE(JSON_EXTRACT(contactsSnapshot, '$.name'))", name);
        queryWrapper.eq(StringUtils.isNotBlank(phone), "JSON_UNQUOTE(JSON_EXTRACT(contactsSnapshot, '$.phone'))", phone);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 校验
     */
    @Override
    public void validClothesRentOrder(ClothesRentOrderAddRequest clothesRentOrderAddRequest) {
        Long clothesId = clothesRentOrderAddRequest.getClothesId();
        Long contactsId = clothesRentOrderAddRequest.getContactsId();
        Integer rentDays = clothesRentOrderAddRequest.getRentDays();
        ThrowUtils.throwIf(ObjectUtils.anyNull(clothesId, contactsId, rentDays), ErrorCode.PARAMS_ERROR);
    }



    /**
     * 向消息队列中发送订单创建的消息
     */
    @Override
    public void sendClothesRentOrderMessage(Long orderId) {
        // 复制 DELAY_MILLIS 到一个新的 ArrayList
        List<Long> newDelayMillis = new ArrayList<>(MqConstant.DELAY_MILLIS);
        // 延迟检查订单状态信息
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(orderId, newDelayMillis);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_CLOTHES_RENT_ORDER_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue);
                    return message;
                });
    }


    /**
     * 向消息队列中发送服装租赁倒计时的消息
     */
    @Override
    public void sendClothesRentPeriodMessage(Long id, Integer days) {
        long seconds = (long) days * 86400;
        List<Long> splitTimeList = TimeSplitterUtils.splitTime(seconds);
        // 延迟检查优惠券是否过期
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(id, splitTimeList);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_CLOTHES_RENT_PERIOD_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue * 1000);
                    return message;
                });
    }


    /**
     * 校验当前服装是否被租赁
     */
    @Override
    public void validIsBeenRented(Long clothesId) {
        QueryWrapper<ClothesRentOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("orderStatus", OrderStatusConstant.PENDING_SHIPMENT, OrderStatusConstant.PENDING_DELIVERY);
        List<ClothesRentOrder> clothesRentOrderList = this.list(queryWrapper);
        for (ClothesRentOrder clothesRentOrder : clothesRentOrderList) {
            Long clothesSnapshotId = clothesRentOrder.getClothesSnapshot().getId();
            if (clothesSnapshotId.equals(clothesId)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "该服装已被租赁");
            }
        }
    }


}
