package com.cultural.heritage.service.order.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.*;
import com.cultural.heritage.model.dto.cart.CartOrderAddRequest;
import com.cultural.heritage.model.dto.cart.CartOrderItemAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderItemAddRequest;
import com.cultural.heritage.model.dto.order.BookingOrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.capital.GeneralGoodSingleBuyAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderItemMainInfoAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.AddressSnapshot;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.enums.GoodTypeEnum;
import com.cultural.heritage.model.vo.coupon.CouponVO;
import com.cultural.heritage.service.good.*;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.order.PendingServiceGoodService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.utils.MultiDelayMessage;
import com.cultural.heritage.utils.OrderNumberUtils;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Service
@Slf4j
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {


    @Resource
    private AddressMapper addressMapper;


    @Resource
    private ContactsMapper contactsMapper;


    @Resource
    private CouponMapper couponMapper;


    @Resource
    private GoodMapper goodMapper;


    @Resource
    private GoodService goodService;


    @Resource
    private OrderItemService orderItemService;


    @Resource
    private CartRecordService cartRecordService;


    @Resource
    private CartExperienceService cartExperienceService;


    @Resource
    private RabbitTemplate rabbitTemplate;


    @Resource
    private CouponService couponService;


    @Resource
    private UserCouponService userCouponService;


    @Resource
    private PendingServiceGoodService pendingServiceGoodService;


    @Resource
    private PendingServiceOrderService pendingServiceOrderService;


    @Resource
    private RedisTemplate<String, Object> redisTemplate;


    // 邮费价格的键
    private static final String POSTAGE_PRICER_KEY = "postagePrice";



    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<Order> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long userId = orderQueryRequest.getUserId();
        Long orderId = orderQueryRequest.getOrderId();
        BigDecimal minTotalAmount = orderQueryRequest.getMinTotalAmount();
        BigDecimal maxTotalAmount = orderQueryRequest.getMaxTotalAmount();
        String orderStatus = orderQueryRequest.getOrderStatus();
        String orderNumber = orderQueryRequest.getOrderNumber();
        Date startTime = orderQueryRequest.getStartTime();
        Date endTime = orderQueryRequest.getEndTime();
        String sortOrder = orderQueryRequest.getSortOrder();
        String sortField = orderQueryRequest.getSortField();
        String orderType = orderQueryRequest.getOrderType();

        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(orderId), "id", orderId);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minTotalAmount), "totalAmount", minTotalAmount);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxTotalAmount), "totalAmount", maxTotalAmount);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.eq(StringUtils.isNotBlank(orderType), "orderType", orderType);
        queryWrapper.between(startTime != null && endTime != null, "createTime", startTime, endTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderNumber), "orderNumber", orderNumber);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 封装订单详细信息请求体
     */
    @Override
    public OrderAddRequest initOrderMainInfo(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        String userName = orderMainInfoAddRequest.getUserName();
        Long addressId = orderMainInfoAddRequest.getAddressId();
        Long contactsId = orderMainInfoAddRequest.getContactsId();
        Long couponId = orderMainInfoAddRequest.getCouponId();
        String orderType = orderMainInfoAddRequest.getOrderType();
        String note = orderMainInfoAddRequest.getNote();
        BigDecimal totalAmount = orderMainInfoAddRequest.getTotalAmount();
        Boolean isPostage = orderMainInfoAddRequest.getIsPostage();
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();

        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(orderType);
        ThrowUtils.throwIf(orderTypeEnum == null, ErrorCode.PARAMS_ERROR);
        AddressSnapshot addressSnapshot = null;
        ContactsSnapshot contactsSnapshot = null;
        CouponSnapshot couponSnapshot = null;

        // 获取订单地址信息
        if (orderTypeEnum.equals(GoodTypeEnum.PRODUCT)) {
            addressSnapshot = getSnapshot(addressId, addressMapper, AddressSnapshot.class);
        }
        // 获取订单联系人信息
        if (orderTypeEnum.equals(GoodTypeEnum.SERVICE)) {
            contactsSnapshot = getSnapshot(contactsId, contactsMapper, ContactsSnapshot.class);
        }
        // 获取订单优惠券信息
        if (couponId != null) {
            UserCoupon userCoupon = userCouponService.getById(couponId);
            ThrowUtils.throwIf(userCoupon == null || userCoupon.getCouponVO().getStatus().equals("已过期"), ErrorCode.OPERATION_ERROR, "用户优惠券不存在或者已过期");
            CouponVO couponVO = userCoupon.getCouponVO();
            couponSnapshot = new CouponSnapshot();
            BeanUtils.copyProperties(couponVO, couponSnapshot);
            couponSnapshot.setId(couponId);
        }

        // 获取邮费
        BigDecimal postage = new BigDecimal("0.00");
        try {
            // 判断是否有邮费
            if (isPostage) {
                String postagePrice = (String) redisTemplate.opsForValue().get(POSTAGE_PRICER_KEY);
                ThrowUtils.throwIf(postagePrice == null, ErrorCode.OPERATION_ERROR, "参数不存在");
                postage = new BigDecimal(postagePrice).setScale(2, RoundingMode.HALF_UP);
            }
        } catch (Exception e) {
            log.info("邮费参数不存在");
        }

        // 封装订单明细信息
        List<OrderItemAddRequest> orderItemAddRequestList = orderItemMainInfoAddRequestList.stream().map(orderItemMainInfoAddRequest -> {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            String reservationDate = orderItemMainInfoAddRequest.getReservationDate();
            String timeSlot = orderItemMainInfoAddRequest.getTimeSlot();
            OrderItemAddRequest orderItemAddRequest = new OrderItemAddRequest();
            // 获取订单商品信息
            GoodSnapshot goodSnapshot = getSnapshot(goodId, goodMapper, GoodSnapshot.class);

            orderItemAddRequest.setGoodSnapshot(goodSnapshot);
            orderItemAddRequest.setPriceSnapshot(goodSnapshot.getPrice());
            orderItemAddRequest.setQuantity(quantity);
            orderItemAddRequest.setItemTotalAmount(goodSnapshot.getPrice().multiply(BigDecimal.valueOf(quantity)));
            orderItemAddRequest.setReservationDate(reservationDate);
            orderItemAddRequest.setTimeSlot(timeSlot);
            return orderItemAddRequest;
        }).toList();

        // 创建订单详细信息请求体
        OrderAddRequest orderAddRequest = new OrderAddRequest();
        orderAddRequest.setUserName(userName);
        orderAddRequest.setOrderType(orderType);
        orderAddRequest.setNote(note);
        orderAddRequest.setTotalAmount(totalAmount);
        orderAddRequest.setPostage(postage);
        orderAddRequest.setAddressSnapshot(addressSnapshot);
        orderAddRequest.setContactsSnapshot(contactsSnapshot);
        orderAddRequest.setCouponSnapshot(couponSnapshot);
        orderAddRequest.setOrderItemList(orderItemAddRequestList);
        return orderAddRequest;
    }



    /**
     * 校验订单
     */
    @Override
    public void validOrder(Order order) {
        String orderType = order.getOrderType();
        String orderNumber = order.getOrderNumber();
        Long userId = order.getUserId();
        String userName = order.getUserName();
        BigDecimal totalAmount = order.getTotalAmount();
        String orderStatus = order.getOrderStatus();
        if (ObjectUtils.isEmpty(totalAmount) || totalAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.isEmpty(userId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isAnyBlank(orderType, orderNumber, userName, orderStatus)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }




    /**
     * 创建通用订单(常规类和服务类商品)
     */
    @Override
    public Long createCommonOrder(OrderMainInfoAddRequest orderMainInfoAddRequest, Long userId, boolean isCartOrder, List<Long> cartIds, boolean isGeneral) {
        // 封装订单详细信息请求体
        OrderAddRequest orderAddRequest = this.initOrderMainInfo(orderMainInfoAddRequest);
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 创建一个总订单
        Order order = new Order();
        BeanUtils.copyProperties(orderAddRequest, order);
        order.setUserId(userId);
        // 生成订单编号
        String orderNumber = OrderNumberUtils.generateOrderId();
        order.setOrderNumber(orderNumber);
        // 设置物流单号
        order.setTrackingNumber("无");
        // 设置订单状态
        order.setOrderStatus(OrderStatusConstant.PENDING_PAYMENT);
        // 校验订单
        this.validOrder(order);
        this.save(order);
        // 创建订单明细
        Long id = order.getId();
        List<OrderItemAddRequest> orderItemList = orderAddRequest.getOrderItemList();
        List<OrderItems> newOrderItemsList = orderItemList.stream().map(orderItem -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(orderItem, orderItems);
            orderItems.setOrderId(id);
            return orderItems;
        }).toList();
        orderItemService.saveBatch(newOrderItemsList);
        if (isGeneral) {
            // 扣减商品库存
            List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();
            this.deductionGeneralGoodInventory(orderItemMainInfoAddRequestList);
        } else {
            // 生成服务类商品待处理订单记录
            this.pendingOrdersForPackagedServiceItems(order);
        }

        // 清空购物车
        if (isCartOrder) {
            if (isGeneral) cartRecordService.removeBatchByIds(cartIds);
            else cartExperienceService.removeBatchByIds(cartIds);
        }
        return id;
    }






    /**
     * 生成服务类商品待处理订单记录
     */
    private void pendingOrdersForPackagedServiceItems (Order order) {
        // 获取map集合（键：服务类商品待处理id, 值：服务类商品待处理记录）
        Map<String, PendingServiceGood> pendingGoodMap = goodService.getPendingServiceGoodMap();

        QueryWrapper<OrderItems> orderItemsQueryWrapper = new QueryWrapper<>();
        orderItemsQueryWrapper.eq("orderId", order.getId());
        List<OrderItems> orderItemsList = orderItemService.list(orderItemsQueryWrapper);

        List<PendingServiceOrder> pendingServiceOrderList = new ArrayList<>();
        for (OrderItems orderItems : orderItemsList) {
            PendingServiceOrder pendingServiceOrder = new PendingServiceOrder();
            pendingServiceOrder.setOrderItemId(orderItems.getId());
            pendingServiceOrder.setOrderNumber(order.getOrderNumber());
            pendingServiceOrder.setUserId(order.getUserId());
            pendingServiceOrder.setName(order.getContactsSnapshot().getName());
            pendingServiceOrder.setPhone(order.getContactsSnapshot().getPhone());
            pendingServiceOrder.setQuantity(orderItems.getQuantity());
            pendingServiceOrder.setItemTotalAmount(orderItems.getItemTotalAmount());
            pendingServiceOrder.setOrderItemStatus(order.getOrderStatus());
            BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest(orderItems.getGoodSnapshot().getId(), orderItems.getReservationDate(), orderItems.getTimeSlot());
            PendingServiceGood pendingServiceGood = pendingGoodMap.get(bookingOrderQueryRequest.toString());
            pendingServiceOrder.setPendingId(pendingServiceGood.getId());

            pendingServiceOrderList.add(pendingServiceOrder);
        }

        pendingServiceOrderService.saveBatch(pendingServiceOrderList);
    }







    /**
     * 扣减常规类商品库存
     */
    private void deductionGeneralGoodInventory(List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList) {

        // 封装map集合（键：商品id, 值：购买数量）
        Map<Long, Integer> map = new HashMap<>();
        for (OrderItemMainInfoAddRequest orderItemMainInfoAddRequest : orderItemMainInfoAddRequestList) {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            map.put(goodId, quantity);
        }
        // 批量修改商品库存信息
        List<Long> goodIds = orderItemMainInfoAddRequestList.stream().map(OrderItemMainInfoAddRequest::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);
        for (Good good : goodList) {
            Integer inventory = map.get(good.getId());
            good.setInventory(good.getInventory() - inventory);
        }
        goodService.updateBatchById(goodList);
    }



    /**
     * 校验单独购买的商品数据是否准确
     */
    @Override
    public void validSingleGoodOrder(GeneralGoodSingleBuyAddRequest generalGoodSingleBuyAddRequest, boolean isGeneral) {
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = generalGoodSingleBuyAddRequest.getOrderItemMainInfoAddRequestList();
        if (orderItemMainInfoAddRequestList.size() != 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "购买了多种商品或未购买商品");
        }
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest = orderItemMainInfoAddRequestList.get(0);
        Long goodId = orderItemMainInfoAddRequest.getGoodId();
        Good good = goodService.getById(goodId);
        // 判断商品是否存在或者下架
        goodService.checkGoodIsExist(good);

        if (isGeneral) {
            goodService.checkInventoryIsEnough(orderItemMainInfoAddRequest.getQuantity(), good.getInventory());
        } else {
            String reservationDate = orderItemMainInfoAddRequest.getReservationDate();
            String timeSlot = orderItemMainInfoAddRequest.getTimeSlot();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            goodService.checkServiceGoodBookingDetailIsProperly(goodId, reservationDate, timeSlot, quantity);
        }
    }





    /**
     * 计算单个商品购买的总金额
     */
    @Override
    public BigDecimal calculateTotalAmount(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest = orderItemMainInfoAddRequestList.get(0);
        // 获取优惠券满减金额
        Long userCouponId = orderMainInfoAddRequest.getCouponId();
        BigDecimal conditionAmount = new BigDecimal("0.00");
        if (userCouponId != null) {
            UserCoupon userCoupon = userCouponService.getById(userCouponId);
            ThrowUtils.throwIf(userCoupon == null || userCoupon.getCouponVO().getStatus().equals("已过期"), ErrorCode.OPERATION_ERROR, "优惠券不存在或已过期");
            conditionAmount = userCoupon.getCouponVO().getConditionAmount();
            // 更新优惠券状态
            userCoupon.setIsUsed(1);
            userCouponService.updateById(userCoupon);
        }
        Integer quantity = orderItemMainInfoAddRequest.getQuantity();
        Long goodId = orderItemMainInfoAddRequest.getGoodId();
        Good good = goodService.getById(goodId);
        return good.getPrice().multiply(BigDecimal.valueOf(quantity)).subtract(conditionAmount);
    }



    /**
     * 向消息队列中发送订单创建的消息
     */
    @Override
    public void sendCreateOrderMessage(Long orderId) {
        // 复制 DELAY_MILLIS 到一个新的 ArrayList
        List<Long> newDelayMillis = new ArrayList<>(MqConstant.DELAY_MILLIS);
        // 延迟检查订单状态信息
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(orderId, newDelayMillis);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_ORDER_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue);
                    return message;
                });
    }


    /**
     * 判断是否是常规类商品
     */
    @Override
    public boolean isGeneralGood(String orderType) {
        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(orderType);
        return GoodTypeEnum.PRODUCT.equals(orderTypeEnum);
    }



    /**
     * 常规类商品购物车封装订单主要信息请求体
     */
    @Override
    public OrderMainInfoAddRequest packagedOrderMainInfoAddRequest(CartOrderAddRequest cartOrderAddRequest) {
        // 封装成订单主要信息请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = new OrderMainInfoAddRequest();
        BeanUtils.copyProperties(cartOrderAddRequest, orderMainInfoAddRequest);
        // 获取购物车id列表
        List<CartOrderItemAddRequest> cartOrderItemAddRequestList = cartOrderAddRequest.getCartOrderItemAddRequestList();
        List<Long> cartIds = cartOrderItemAddRequestList.stream().map(CartOrderItemAddRequest::getCartRecordId).toList();
        // 根据id列表获取购物车商品信息
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", cartIds);
        List<CartRecord> cartRecordList = cartRecordService.list(queryWrapper);

        List<Long> goodIds = cartRecordList.stream().map(CartRecord::getGoodId).toList();
        QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(goodQueryWrapper);
        // 封装map集合（键：商品id, 值：商品信息）
        Map<Long, Good> goodMap = new HashMap<>();
        for (Good good : goodList) {
            goodMap.put(good.getId(), good);
        }
        // 封装成一个map集合（键：购物车id, 值：购物车商品数量）
        Map<Long, Integer> map = new HashMap<>();
        for (CartOrderItemAddRequest cartOrderItemAddRequest : cartOrderItemAddRequestList) {
            Long cartRecordId = cartOrderItemAddRequest.getCartRecordId();
            Integer quantity = cartOrderItemAddRequest.getQuantity();
            map.put(cartRecordId, quantity);
        }

        // 计算订单总金额
        BigDecimal totalAmount = BigDecimal.ZERO;
        for (CartRecord cartRecord : cartRecordList) {
            Long goodId = cartRecord.getGoodId();
            Good good = goodMap.get(goodId);
            Long cartRecordId = cartRecord.getId();
            Integer quantity = map.get(cartRecordId);
            totalAmount = totalAmount.add(good.getPrice().multiply(BigDecimal.valueOf(quantity)));
        }
        // 处理优惠券满减金额
        Long couponId = orderMainInfoAddRequest.getCouponId();
        if (couponId != null) {
            UserCoupon userCoupon = userCouponService.getById(couponId);
            ThrowUtils.throwIf(userCoupon == null || userCoupon.getCouponVO().getStatus().equals("已过期"), ErrorCode.OPERATION_ERROR, "用户优惠券不存在或已过期");
            BigDecimal conditionAmount = userCoupon.getCouponVO().getConditionAmount();
            totalAmount = totalAmount.subtract(conditionAmount);
            // 更新优惠券状态
            userCoupon.setIsUsed(1);
            userCouponService.updateById(userCoupon);
        }
        // 填充订单主要信息请求体金额
        orderMainInfoAddRequest.setTotalAmount(totalAmount);
        // 封装成订单明细主要信息请求体列表
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = new ArrayList<>();
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest;
        for (CartRecord cartRecord : cartRecordList) {
            Long goodId = cartRecord.getGoodId();
            Long id = cartRecord.getId();
            Integer quantity = map.get(id);
            orderItemMainInfoAddRequest = new OrderItemMainInfoAddRequest(goodId, quantity, null, null);
            orderItemMainInfoAddRequestList.add(orderItemMainInfoAddRequest);
        }
        // 填充到订单主要信息请求体
        orderMainInfoAddRequest.setOrderItemMainInfoAddRequestList(orderItemMainInfoAddRequestList);
        return orderMainInfoAddRequest;
    }




    /**
     * 服务类商品购物车封装订单主要信息请求体
     */
    @Override
    public OrderMainInfoAddRequest packagedOrderMainInfoAddRequest(CartExperienceOrderAddRequest cartExperienceOrderAddRequest) {
        // 封装成订单主要信息请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = new OrderMainInfoAddRequest();
        BeanUtils.copyProperties(cartExperienceOrderAddRequest, orderMainInfoAddRequest);
        // 获取购物车id列表
        List<CartExperienceOrderItemAddRequest> cartExperienceOrderItemAddRequestList = cartExperienceOrderAddRequest.getCartOrderItemAddRequestList();
        List<Long> cartIds = cartExperienceOrderItemAddRequestList.stream().map(CartExperienceOrderItemAddRequest::getCartExperienceId).toList();
        // 根据id列表获取购物车商品信息
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", cartIds);
        List<CartExperience> cartExperienceList = cartExperienceService.list(queryWrapper);

        List<Long> goodIds = cartExperienceList.stream().map(CartExperience::getGoodId).toList();
        QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(goodQueryWrapper);
        // 封装map集合（键：商品id, 值：商品信息）
        Map<Long, Good> goodMap = new HashMap<>();
        for (Good good : goodList) {
            goodMap.put(good.getId(), good);
        }
        // 封装成一个map集合（键：购物车id, 值：购物车商品数量）
        Map<Long, Integer> map = new HashMap<>();
        for (CartExperienceOrderItemAddRequest cartExperienceOrderItemAddRequest : cartExperienceOrderItemAddRequestList) {
            Long cartExperienceId = cartExperienceOrderItemAddRequest.getCartExperienceId();
            Integer quantity = cartExperienceOrderItemAddRequest.getQuantity();
            map.put(cartExperienceId, quantity);
        }

        // 计算订单总金额
        BigDecimal totalAmount = BigDecimal.ZERO;
        for (CartExperience cartExperience : cartExperienceList) {
            Long goodId = cartExperience.getGoodId();
            Good good = goodMap.get(goodId);
            Long cartExperienceId = cartExperience.getId();
            Integer quantity = map.get(cartExperienceId);
            totalAmount = totalAmount.add(good.getPrice().multiply(BigDecimal.valueOf(quantity)));
        }
        // 处理优惠券满减金额
        Long couponId = orderMainInfoAddRequest.getCouponId();
        if (couponId != null) {
            UserCoupon userCoupon = userCouponService.getById(couponId);
            ThrowUtils.throwIf(userCoupon == null || userCoupon.getCouponVO().getStatus().equals("已过期"), ErrorCode.OPERATION_ERROR, "用户优惠券不存在或已过期");
            BigDecimal conditionAmount = userCoupon.getCouponVO().getConditionAmount();
            totalAmount = totalAmount.subtract(conditionAmount);
            // 更新优惠券状态
            userCoupon.setIsUsed(1);
            userCouponService.updateById(userCoupon);
        }
        // 填充订单主要信息请求体金额
        orderMainInfoAddRequest.setTotalAmount(totalAmount);
        // 封装成订单明细主要信息请求体列表
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = new ArrayList<>();
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest;
        for (CartExperience cartExperience : cartExperienceList) {
            Long goodId = cartExperience.getGoodId();
            Long cartExperienceId = cartExperience.getId();
            Integer quantity = map.get(cartExperienceId);
            String reservationDate = cartExperience.getReservationDate();
            String timeSlot = cartExperience.getTimeSlot();
            orderItemMainInfoAddRequest = new OrderItemMainInfoAddRequest(goodId, quantity, reservationDate, timeSlot);
            orderItemMainInfoAddRequestList.add(orderItemMainInfoAddRequest);
        }
        // 填充到订单主要信息请求体
        orderMainInfoAddRequest.setOrderItemMainInfoAddRequestList(orderItemMainInfoAddRequestList);
        return orderMainInfoAddRequest;
    }


    /**
     * 根据id获取订单(地址，联系人，商品)信息
     */
    private static <T, S> S getSnapshot(Long id, BaseMapper<T> mapper, Class<S> snapshotClass) {
        // 创建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        // 查询实体对象
        T entity = mapper.selectOne(queryWrapper);
        // 判断实体对象是否存在，不存在则抛出异常
        ThrowUtils.throwIf(entity == null, ErrorCode.NOT_FOUND_ERROR);
        try {
            // 创建快照对象实例
            S snapshot = snapshotClass.getDeclaredConstructor().newInstance();
            // 复制属性到快照对象
            BeanUtils.copyProperties(entity, snapshot);
            return snapshot;
        } catch (Exception e) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR);
        }
    }



}
